import { IJSONComment, IJSONCommentConfiguration, IReplacer } from './types';
/**
 * Base writer to add comments to stringified JSON.
 * JSON stringify implementation is based on the following code:
 * https://github.com/douglascrockford/JSON-js/blob/2a76286e00cdc1e98fbc9e9ec6589563a3a4c3bb/json2.js
 *
 * @param CommentDataNodeType Subclasses should use this type of node to track current node and related data.
 */
export declare abstract class JSONCommentWriterBase<CommentDataNodeType> {
    private static readonly defaultConfiguration;
    private readonly configuration;
    private indent;
    private replacer;
    private path;
    /**
     * Construct a new JSONCommentWriter.
     * @param configuration Styling configuration for comments.
     */
    constructor(configuration?: Partial<IJSONCommentConfiguration>);
    /**
     * Convert the given object to a JSON string with comments specified eariler.
     * @param object The root object to convert to JSON string.
     * @param replacer A function that alters the behavior of the stringification process,
     * or an array of String and Number objects that serve as a whitelist for
     * selecting/filtering the properties of the value object to be included in the JSON string.
     *
     * If this value is null or not provided,
     * all properties of the object are included in the resulting JSON string.
     * @param space A String or Number object that's used to insert white space into
     * the output JSON string for readability purposes.
     *
     * If this is a Number, it indicates the number of space characters to use as white space;
     * this number is capped at 10 (if it is greater, the value is just 10).
     * Values less than 1 indicate that no space should be used.
     *
     * If this is a String, the string is used as white space.
     * If this parameter is not provided (or is null), no white space is used.
     *
     * @returns The JSON string, or `undefined` if object is `undefined`.
     */
    stringify(object: any, replacer?: IReplacer, space?: number | string): string | undefined;
    protected abstract readonly root: Readonly<CommentDataNodeType>;
    protected abstract nextNode(currentNode: Readonly<CommentDataNodeType>, key: string | number): Readonly<CommentDataNodeType> | undefined;
    protected abstract getComments(currentNode: Readonly<CommentDataNodeType>): IJSONComment[];
    private renderComment;
    private getChildJSON;
    /**
     * Convert holder[key] to JSON.
     * @param key The key of the item to be serialized.
     * Note that it accepts number to allow for JS engine optimization.
     * @param holder The object holding the item.
     * @param gap Accumulated indent.
     * @param nodes Selecter tree nodes matched for the item
     * @returns JSON representation of the item.
     * `undefined` if the item is `undefined`.
     */
    private objToJSON;
}
