"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var wrap = require("word-wrap");
/**
 * Base writer to add comments to stringified JSON.
 * JSON stringify implementation is based on the following code:
 * https://github.com/douglascrockford/JSON-js/blob/2a76286e00cdc1e98fbc9e9ec6589563a3a4c3bb/json2.js
 *
 * @param CommentDataNodeType Subclasses should use this type of node to track current node and related data.
 */
var JSONCommentWriterBase = /** @class */ (function () {
    /**
     * Construct a new JSONCommentWriter.
     * @param configuration Styling configuration for comments.
     */
    function JSONCommentWriterBase(configuration) {
        this.indent = '';
        this.path = [];
        this.configuration = tslib_1.__assign({}, JSONCommentWriterBase.defaultConfiguration, configuration);
    }
    /**
     * Convert the given object to a JSON string with comments specified eariler.
     * @param object The root object to convert to JSON string.
     * @param replacer A function that alters the behavior of the stringification process,
     * or an array of String and Number objects that serve as a whitelist for
     * selecting/filtering the properties of the value object to be included in the JSON string.
     *
     * If this value is null or not provided,
     * all properties of the object are included in the resulting JSON string.
     * @param space A String or Number object that's used to insert white space into
     * the output JSON string for readability purposes.
     *
     * If this is a Number, it indicates the number of space characters to use as white space;
     * this number is capped at 10 (if it is greater, the value is just 10).
     * Values less than 1 indicate that no space should be used.
     *
     * If this is a String, the string is used as white space.
     * If this parameter is not provided (or is null), no white space is used.
     *
     * @returns The JSON string, or `undefined` if object is `undefined`.
     */
    JSONCommentWriterBase.prototype.stringify = function (object, replacer, space) {
        this.indent = '';
        if (typeof space === 'number') {
            for (var i = 0; i < space; i++) {
                this.indent += ' ';
            }
        }
        else if (typeof space === 'string') {
            this.indent = space;
        }
        this.replacer = replacer;
        if (replacer && typeof replacer !== 'function' && !Array.isArray(replacer)) {
            throw new Error('Argument `replacer` is expected to be a function or an array');
        }
        this.path = [];
        var _a = this.getChildJSON({ '': object }, '', '', this.root), parts = _a.comments, childJSON = _a.childJSON, lineEndComment = _a.lineEndComment;
        if (childJSON === undefined) {
            return undefined;
        }
        parts.push(childJSON + (lineEndComment || ''));
        return parts.join('\n');
    };
    JSONCommentWriterBase.prototype.renderComment = function (gap, path, comment) {
        var content = typeof comment.content === 'function' ? comment.content(path.slice(1)) : comment.content;
        if (content === undefined) {
            return undefined;
        }
        // If there's indent or is root level
        if (gap || this.path.length === 1) {
            if (this.configuration.maxLineLength > 0) {
                content = wrap(content, { width: this.configuration.maxLineLength, indent: '', trim: true });
            }
            if (comment.type === 'block') {
                content = content.replace(/\*\//g, '*\/');
                if (this.configuration.styledBlockComment) {
                    return "/**\n" + gap + " * " + content.replace(/\n/g, "\n" + gap + " * ") + "\n" + gap + " */";
                }
                else if (this.configuration.spaceAroundCommentSymbol) {
                    return "/* " + content.replace(/\n/g, "\n" + gap) + " */";
                }
                else {
                    return "/*" + content.replace(/\n/g, "\n" + gap) + "*/";
                }
            }
            else {
                if (comment.type === 'end' && content.includes('\n')) {
                    throw new Error('Comment of type `end` is expected to be single-line');
                }
                if (this.configuration.spaceAroundCommentSymbol) {
                    return "// " + content.replace(/\n/g, "\n" + gap + "// ");
                }
                else {
                    return "//" + content.replace(/\n/g, "\n" + gap + "//");
                }
            }
        }
        else {
            if (comment.type === 'block') {
                if (content.includes('\n')) {
                    throw new Error('Comment is expected to be single-line when space is 0');
                }
                content = content.replace(/\*\//g, '*\/');
                if (this.configuration.spaceAroundCommentSymbol) {
                    return "/* " + content + " */";
                }
                else {
                    return "/*" + content + "*/";
                }
            }
            else {
                return undefined;
            }
        }
    };
    JSONCommentWriterBase.prototype.getChildJSON = function (value, nextKey, gap, node) {
        var nextNode = node && this.nextNode(node, nextKey);
        var parts = [];
        var lineEndComment;
        this.path.push(nextKey);
        if (nextNode) {
            for (var _i = 0, _a = this.getComments(nextNode); _i < _a.length; _i++) {
                var comment = _a[_i];
                if (comment.type === 'end' && lineEndComment !== undefined) {
                    throw new Error('Comment of type `end` is expected to be unique for each field');
                }
                var commentString = this.renderComment(gap, this.path, comment);
                if (comment.type === 'end') {
                    lineEndComment = commentString;
                }
                else if (commentString !== undefined) {
                    parts.push(commentString);
                }
            }
        }
        var childJSON = this.objToJSON(nextKey, value, gap, nextNode);
        this.path.pop();
        if (lineEndComment !== undefined && this.configuration.spaceAroundCommentSymbol) {
            lineEndComment = ' ' + lineEndComment;
        }
        return { comments: parts, childJSON: childJSON, lineEndComment: lineEndComment };
    };
    /**
     * Convert holder[key] to JSON.
     * @param key The key of the item to be serialized.
     * Note that it accepts number to allow for JS engine optimization.
     * @param holder The object holding the item.
     * @param gap Accumulated indent.
     * @param nodes Selecter tree nodes matched for the item
     * @returns JSON representation of the item.
     * `undefined` if the item is `undefined`.
     */
    // tslint:disable-next-line
    JSONCommentWriterBase.prototype.objToJSON = function (key, holder, gap, node) {
        var value = holder[key];
        if (value && typeof value === 'object' && typeof value.toJSON === 'function') {
            value = value.toJSON(key);
        }
        if (typeof this.replacer === 'function') {
            value = this.replacer.call(holder, key, value);
        }
        if (typeof value === 'object') {
            if (!value) {
                return 'null';
            }
            var currGap = gap + this.indent;
            var lineBreakCurrGap = currGap ? '\n' + currGap : '';
            var lineEndComments_1 = {};
            var partial_1 = [];
            var fnPartialToLine = function (p, i) { return "" + p + (i < partial_1.length - 1 ? ',' : '') + (lineEndComments_1[i] || ''); };
            if (Array.isArray(value)) {
                if (value.length === 0) {
                    return '[]';
                }
                for (var i = 0; i < value.length; i++) {
                    var _a = this.getChildJSON(value, i, currGap, node), parts = _a.comments, childJSON = _a.childJSON, lineEndComment = _a.lineEndComment;
                    parts.push(childJSON || 'null');
                    if (lineEndComment !== undefined) {
                        lineEndComments_1[i] = lineEndComment;
                    }
                    partial_1.push(parts.join(lineBreakCurrGap));
                }
                return currGap ? "[\n" + currGap + partial_1.map(fnPartialToLine).join(lineBreakCurrGap) + "\n" + gap + "]" : "[" + partial_1.join(',') + "]";
            }
            else {
                var keys = Array.isArray(this.replacer) ? this.replacer : Object.keys(value);
                if (keys.length === 0) {
                    return '{}';
                }
                for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
                    var k = keys_1[_i];
                    var _b = this.getChildJSON(value, k, currGap, node), parts = _b.comments, childJSON = _b.childJSON, lineEndComment = _b.lineEndComment;
                    if (childJSON) {
                        parts.push(JSON.stringify(k) + (currGap ? ': ' : ':') + childJSON);
                        if (lineEndComment !== undefined) {
                            lineEndComments_1[partial_1.length] = lineEndComment;
                        }
                        partial_1.push(parts.join(lineBreakCurrGap));
                    }
                }
                return currGap ? "{\n" + currGap + partial_1.map(fnPartialToLine).join(lineBreakCurrGap) + "\n" + gap + "}" : "{" + partial_1.join(',') + "}";
            }
        }
        else {
            return JSON.stringify(value);
        }
    };
    JSONCommentWriterBase.defaultConfiguration = {
        spaceAroundCommentSymbol: true,
        styledBlockComment: true,
        maxLineLength: 80
    };
    return JSONCommentWriterBase;
}());
exports.JSONCommentWriterBase = JSONCommentWriterBase;
//# sourceMappingURL=jsonCommentWriterBase.js.map